// ChildView.cpp : implementation of the CChildView class
//
#include "stdafx.h"
#include "FDExtCapture.h"
#include "ChildView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CChildView
BEGIN_MESSAGE_MAP(CChildView, CWnd)
	ON_WM_PAINT()
	ON_WM_TIMER()
	ON_WM_DESTROY()
	ON_COMMAND(ID_FILE_PROPERTIES, &CChildView::OnFileProperties)
END_MESSAGE_MAP()

BOOL CChildView::PreCreateWindow(CREATESTRUCT& cs) 
{
	if (!CWnd::PreCreateWindow(cs))
		return FALSE;
	cs.dwExStyle |= WS_EX_CLIENTEDGE;
	cs.style &= ~WS_BORDER;
	cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, 
		::LoadCursor(NULL, IDC_ARROW), reinterpret_cast<HBRUSH>(COLOR_WINDOW+1), NULL);
	return TRUE;
}

void CChildView::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	RECT rc;  GetClientRect(&rc);
	if( m_pImage != NULL ) {
		::SetDIBitsToDevice(dc, 0,0,m_bmpi.biWidth,m_bmpi.biHeight, 0,0,0,m_bmpi.biHeight, m_pImage, (BITMAPINFO*)&m_bmpi, DIB_RGB_COLORS);
		dc.Rectangle(m_bmpi.biWidth,0, rc.right,m_bmpi.biHeight);
		dc.Rectangle(0,m_bmpi.biHeight, rc.right,rc.bottom);
		CString str;  str.Format(_T("%d"),m_nNextFrame);
		dc.TextOut(10,m_bmpi.biHeight+10, str);
	} else {
		dc.Rectangle(0,0, rc.right,rc.bottom);
		dc.TextOut(10,10, _T("Choose menu \"Video Input\""));
	}
}

const int IDC_BASE1 = 1000;
const int IDC_BASE2 = 2000;
const int IDC_BASE3 = 3000;
void CChildView::OnFileProperties()
{
	// create board manager
	CComPtr<IFDExtMgr> pIMgr;
	HRESULT hres = pIMgr.CoCreateInstance(CLSID_FDExtMgr);
	ATLASSERT(hres==S_OK);
	// scan all valid objects for capture from input (FDObjType_VideoSrc)
	long nIndex = 0;
	CFDObjectInfo info;
	HMENU hMenu = CreatePopupMenu();
	while (pIMgr->get_NextValidFDObjectInfo(&nIndex, FDObjType_VideoSrc, &info) == S_OK) {
		// add name to local menu with ID by index of the object (IDC_BASE1 + index)
		WCHAR name[64];  wsprintf(name, L"%s/%s", info.m_BoardName, info.m_FDObjFriendlyName);
		AppendMenu(hMenu, MF_STRING, IDC_BASE1 + nIndex - 1, name);
	}
	AppendMenu(hMenu, MF_SEPARATOR, 0, _T(""));
	// scan all valid objects for capture from output (FDObjType_VideoMixSrc)
	nIndex = 0;
	while (pIMgr->get_NextValidFDObjectInfo(&nIndex, FDObjType_VideoMixSrc, &info) == S_OK) {
		// add name to local menu with ID by index of the object (IDC_BASE2 + index)
		WCHAR name[64];  wsprintf(name, L"%s/%s", info.m_BoardName, info.m_FDObjFriendlyName);
		AppendMenu(hMenu, MF_STRING, IDC_BASE2 + nIndex - 1, name);
	}
	POINT pt;  GetCursorPos(&pt);
	int id = TrackPopupMenu(hMenu, TPM_RETURNCMD | TPM_LEFTALIGN | TPM_TOPALIGN | TPM_RIGHTBUTTON, pt.x + 10, pt.y, 0, m_hWnd, NULL);
	if (id < IDC_BASE1 || id > IDC_BASE3) {
		Invalidate(TRUE);
		return;
	}
	// try select capturer
	CComPtr<IUnknown> pIUnk;
	if (id < IDC_BASE2) { // capture from input
		hres = pIMgr->get_FDObjectByIndex(id-IDC_BASE1, FDObjType_VideoSrc, &pIUnk);
	} else { // capture from output
		hres = pIMgr->get_FDObjectByIndex(id - IDC_BASE2, FDObjType_VideoMixSrc, &pIUnk);
	}
	ATLASSERT(hres==S_OK);
	// obtain capture interface
	m_pICapture = NULL;
	hres = pIUnk.QueryInterface(&m_pICapture);
	ATLASSERT(hres==S_OK);
	// obtain video info
	CBoardVideoParams vInfo;
	hres = m_pICapture->FDGetVideoParams(&vInfo);
	ATLASSERT(hres==S_OK);
	ZeroMemory(&m_bmpi,sizeof(m_bmpi));
	m_bmpi.biSize = sizeof(m_bmpi);
	m_bmpi.biBitCount = 32;
	m_bmpi.biCompression = BI_RGB;
	m_bmpi.biHeight = vInfo.m_DY;
	m_bmpi.biWidth = vInfo.m_DX;
	m_bmpi.biPlanes = 1;
	m_bmpi.biSizeImage = vInfo.m_DY * vInfo.m_DX * 4;
	// allocate image for capture whole frame
	m_pImage = new BYTE[m_bmpi.biSizeImage];
	ATLASSERT(m_pImage != NULL);
	// obtain current frame (as starting frame number)
	ULONGLONG tmp;
	hres = m_pICapture->FDGetVideoTime(&m_nNextFrame,&tmp);
	ATLASSERT(hres==S_OK);
	// all right - start playback
	SetTimer(101,10,NULL);
}

void CChildView::OnTimer(UINT_PTR nIDEvent)
{
	if( m_pICapture==NULL )  return;
	// try to read frame with number 'm_nNextFrame' from capture buffer
	ULONGLONG tmp, n = m_nNextFrame;
	HRESULT hres = m_pICapture->FDReadVideoFrameData(m_pImage, m_bmpi.biSizeImage, m_bmpi.biWidth*4, &n, &tmp, FDCaptureVideoFormat_BGRA);
	ATLASSERT(SUCCEEDED(hres));
	if( hres==S_FALSE ) {
		// frame is not ready yet
		return;
	}
	// frame 'n' was captured - next time will capture next frame
	m_nNextFrame = n + 1;
	// draw captured image
	Invalidate(FALSE);
}
